/************************************************************************
 * IRC - Internet Relay Chat, src/help.c
 * Copyright (C) 2001 Alasdair McWilliam. RageIRCd, Inc.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 1, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

/*
 * RageIRCd v1.5.0 (pandora)
 */

#include "struct.h"
#include "common.h"
#include "sys.h"
#include "h.h"
#include "dynconf2.h"
#include "numeric.h"

#define HDR(string)	sendto_one(sptr, ":%s 290 %s :%s", me.name, name, string);
#define SND(string)	sendto_one(sptr, ":%s 291 %s :%s", me.name, name, string);
#define FTR(string)	sendto_one(sptr, ":%s 292 %s :%s", me.name, name, string);
#define HLP(string)	sendto_one(sptr, ":%s 293 %s :%s", me.name, name, string);

int	m_help(cptr, sptr, parc, parv)
	aClient		*cptr, *sptr;
	int		parc;
	char		*parv[];
{
	char		*message;
	int		sent;

	message = ((parc > 1) ? parv[1] : NULL);
	sent = 0;

	if (message != NULL)
	{
		if (message[0] == '?')
		{
			parse_help(sptr, parv[0], message + 1);
			sent++;
		}
		else if (message[0] == '!')
		{
			*message++;
			if (*message == '\0')
			{
				sendto_one(sptr, err_str(ERR_NOTEXTTOSEND), me.name, parv[0]);
				return 0;
			}

			sendto_umode(UMODE_HELPOP, "*** HelpOp -- from %s: %s", parv[0], message);
			if (MyClient(sptr))
			{
				sendto_one(sptr, rpl_str(RPL_SENTHELPOP), me.name, parv[0]);
			}

			*message--;
			sendto_serv_butone(cptr, ":%s HELP :%s", parv[0], message);

			sent++;
		}
	}

	if (sent == 0)
	{
		message = "";
		parse_help(sptr, parv[0], message);
	}

	return 0;
}

int	parse_help(sptr, name, help)
	aClient		*sptr;
	char		*name, *help;
{
	if (!myncmp(help, "USERCMDS", 9))
	{
		HDR("*** RageIRCd Help System - User Commands ***");
		SND(" ");
		SND("Below is a list of User Commands currently available. Type \'/HELPOP <command>\'");
		SND("for more information on the command.");
		SND(" ");
		SND("NICK     WHOIS    WHO     RULES     IDENTIFY   PONG");
		SND("ISON     JOIN     PART    NAMES     USERHOST");
		SND("LUSERS   MAP      QUIT    NEWIDENT  ABUSE");
		SND("STATS    LINKS    ADMIN   NEWNAME   MKPASSWD");
		SND("INVITE   KICK     AWAY    WATCH     LIST");
		SND("PRIVMSG  NOTICE   KNOCK   VHOST     TOPIC");
		SND("MODE     INFO     TIME    VERSION   CHANSTATS");
		SND("MOTD     SILENCE  WHOWAS  NEWS      PING");
	}
	else if (!myncmp(help, "NICK", 9))
	{
		HDR("*** RageIRCd Help System - User Commands - NICK ***");
		SND(" ");
		SND("Changes your \"online identify\" (nickname) on the server. All channels that you are");
		SND("a member of will be alerted of your nickname change.");
		SND(" ");
		SND("Example: /NICK <newnickname>");
		SND("         /NICK [-Th3Dud3-]");
	}
	else if (!myncmp(help, "WHOIS", 9))
	{
		HDR("*** RageIRCd Help System - User Commands - WHOIS ***");
		SND(" ");
		SND("Shows you information about the user you specify. The information includes nickname,");
		SND("username, hostname, real name, etc.");
		SND(" ");
		SND("Example: /WHOIS Jamie");
		SND("         /WHOIS [-Th3Dud3-]");
	}
	else if (!myncmp(help, "WHO", 9))
	{
		HDR("*** RageIRCd Help System - User Commands - WHO ***");
		SND(" ");
		SND("Searches visible (-i) user information for the information provided. IRC Operators");
		SND("are also able to search invisible (+i) users. When used on a channel, it will give");
		SND("a list of all the visible users on the specified channel.");
		SND(" ");
		SND(" Example: /WHO *.btinternet.com");
		SND("          /WHO [-Th3Dud3-]");
		SND("          /WHO #Turbo-IRC");
	}
	else if (!myncmp(help, "RULES", 9))
	{
		HDR("*** RageIRCd Help System - User Commands - RULES ***");
		SND(" ");
		SND("Reads the Server Rules file (ircd.rules) and displays them to the user.");
		SND(" ");
		SND("Example: /RULES");
		SND("         /RULES HiTech.*");
	}
	else if (!myncmp(help, "IDENTIFY", 9))
	{
		HDR("*** RageIRCd Help System - User Commands - IDENTIFY ***");
		SND(" ");
		SND("A services alias which allows you to identify your nickname with NickServ or channel");
		SND("with ChanServ without anyone being able to \'snoop\'.");
		SND(" ");
		SND("Example: /IDENTIFY <password> - For identifying with NickServ");
		SND("         /IDENTIFY <channel> <password> - For identifying with ChanServ");
	}
	else if (!myncmp(help, "CHANSTATS", 9))
	{
		HDR("*** RageIRCd Help System - User Commands - CHANSTATS ***");
		SND(" ");
		SND("Displays statistical information about the specified channel. The information includes");
		SND("channel modes, channel mode parameters, etc.");
		SND(" ");
		SND("Example: /CHANSTATS #Turbo-IRC");
	}
	else if (!myncmp(help, "ISON", 9))
	{
		HDR("*** RageIRCd Help System - User Commands - ISON ***");
		SND(" ");
		SND("Used to determine whether the specified user is currently connected to IRC.");
		SND(" ");
		SND("Example: /ISON [-Th3Dud3-]");
	}
	else if (!myncmp(help, "JOIN", 9))
	{
		HDR("*** RageIRCd Help System - User Commands - JOIN ***");
		SND(" ");
		SND("Used to enter the specified channel(s). Occupants of the specified channel(s) will be");
		SND("notified of your arrival. Using 0 as the parameter will make you part all channels.");
		SND(" ");
		SND("Example: /JOIN #Turbo-IRC");
		SND("         /JOIN #Turbo-IRC,#RageIRCd,#Turbo");
		SND("         /JOIN 0");
	}
	else if (!myncmp(help, "PART", 9))
	{
		HDR("*** RageIRCd Help System - User Commands - PART ***");
		SND(" ");
		SND("Used to part the specified channel(s). Occupants of the specified channel(s) will be");
		SND("notified or your departure.");
		SND(" ");
		SND("Example: /PART #Turbo-IRC");
		SND("         /PART #Turbo-IRC,#RageIRCd,#Turbo");
	}
	else if (!myncmp(help, "NAMES", 9))
	{
		HDR("*** RageIRCd Help System - User Commands - NAMES ***");
		SND(" ");
		SND("Provides you with a list of all clients which currently occupy the specified channel.");
		SND(" ");
		SND("Example: /NAMES #Turbo-IRC");
	}
	else if (!myncmp(help, "USERHOST", 9))
	{
		HDR("*** RageIRCd Help System - User Commands - USERHOST ***");
		SND(" ");
		SND("Returns the hostname of the speicifed user.");
		SND(" ");
		SND("Example: /USERHOST [-Th3Dud3-]");
	}
	else if (!myncmp(help, "LUSERS", 9))
	{
		HDR("*** RageIRCd Help System - User Commands - LUSERS ***");
		SND(" ");
		SND("Displays the local and global user statistics. The information includes maximum local");
		SND("user count, maximum global user count, etc.");
		SND(" ");
		SND("Example: /LUSERS");
		SND("         /LUSERS HiTech.*");
	}
	else if (!myncmp(help, "MAP", 9))
	{
		HDR("*** RageIRCd Help System - User Commands - MAP ***");
		SND(" ");
		SND("Displays a graphical map of the IRC Network, and the amount of users on each server.");
		SND(" ");
		SND("Example: /MAP");
	}
	else if (!myncmp(help, "QUIT", 9))
	{
		HDR("*** RageIRCd Help System - User Commands - QUIT ***");
		SND(" ");
		SND("Disconnects you from the IRC Network. Clients occupying the channels you are in will be");
		SND("notified of your departure, aswell as IRC Operators on your local server.");
		SND(" ");
		SND("Example: /QUIT");
		SND("         /QUIT Goodbye everyone!");
	}
	else if (!myncmp(help, "NEWIDENT", 9))
	{
		HDR("*** RageIRCd Help System - User Commands - NEWIDENT ***");
		SND(" ");
		SND("Changes your ident (or \'username\') to the new ident you specify. Limit of 9 characters.");
		SND(" ");
		SND("Example: /NEWIDENT alasdair");
	}
	else if (!myncmp(help, "ABUSE", 9))
	{
		HDR("*** RageIRCd Help System - User Commands - ABUSE ***");
		SND(" ");
		SND("Used to report abuse reports to any Abuse Administrators currently connected to IRC.");
		SND(" ");
		SND("Example: /ABUSE :[-Th3Dud3-] is killing me for no reason - help!");
	}
	else if (!myncmp(help, "STATS", 9))
	{
		HDR("*** RageIRCd Help System - User Commands - STATS ***");
		SND(" ");
		SND("Reports statistical information about the specified subject. Some characters are limited");
		SND("to IRC Operators.");
		SND(" ");
		SND("Example: /STATS ?");
		SND("         /STATS w");
		SND("         /STATS k");
	}
	else if (!myncmp(help, "LINKS", 9))
	{
		HDR("*** RageIRCd Help System - User Commands - LINKS ***");
		SND(" ");
		SND("Displays a list of IRC Servers linked to the network.");
		SND(" ");
		SND("Example: /LINKS");
	}
	else if (!myncmp(help, "ADMIN", 9))
	{
		HDR("*** RageIRCd Help System - User Commands - ADMIN ***");
		SND(" ");
		SND("Displays information about the Server Administrator. The information includes the");
		SND("administrators nickname, real name and e-mail address.");
		SND(" ");
		SND("Example: /ADMIN");
	}
	else if (!myncmp(help, "NEWNAME", 9))
	{
		HDR("*** RageIRCd Help System - User Commands - NEWNAME ***");
		SND(" ");
		SND("Changes your 'real name' to the specified name.");
		SND(" ");
		SND("Example: /NEWNAME :Alasdair McWilliam");
	}
	else if (!myncmp(help, "TIME", 9))
	{
		HDR("*** RageIRCd Help System - User Commands - TIME ***");
		SND(" ");
		SND("Displays the current Time and Date of the IRC Server.");
		SND(" ");
		SND("Example: /TIME");
	}
	else if (!myncmp(help, "INVITE", 9))
	{
		HDR("*** RageIRCd Help System - User Commands - INVITE ***");
		SND(" ");
		SND("Sends the specified user an invitation to join the channel. You must be a channel");
		SND("operator or half-operator to be able to use this function.");
		SND(" ");
		SND("Example: /INVITE [-Th3Dud3-] #RageIRCd");
	}
	else if (!myncmp(help, "KICK", 9))
	{
		HDR("*** RageIRCd Help System - User Commands - KICK ***");
		SND(" ");
		SND("Kicks (removes) the specified user from the channel. You must be a channel operator or");
		SND("half-operator to use this function. If no reason is specified, reason becomes \'no reason");
		SND("specified\'.");
		SND(" ");
		SND("Example: /KICK [-Th3Dud3-] #RageIRCd");
		SND("         /KICK kmw #Turbo-IRC bad language");
	}
	else if (!myncmp(help, "AWAY", 9))
	{
		HDR("*** RageIRCd Help System - User Commands - AWAY ***");
		SND(" ");
		SND("Sets your online status to \'away\' because of the specified reason.");
		SND(" ");
		SND("Example: /AWAY I am currently away right now!");
	}
	else if (!myncmp(help, "WATCH", 9))
	{
		HDR("*** RageIRCd Help System - User Commands - WATCH ***");
		SND(" ");
		SND("WATCH is a new type of \'notify\' system. The server will notify you whenever a client");
		SND("in your WATCH list connects or disconnects from IRC. Please note, your WATCH list _DOES_NOT_");
		SND("remain between IRC sessions. You will need to re-build it whenever you reconnect to IRC.");
		SND(" ");
		SND("Example: /WATCH +nick1 +nick2 +nick3");
		SND("         /WATCH -nick1 -nick3");
	}
	else if (!myncmp(help, "LIST", 9))
	{
		HDR("*** RageIRCd Help System - User Commands - LIST ***");
		SND(" ");
		SND("Displays a list of channels currently open. The information includes the current number of");
		SND("channel users, and the channel topic.");
		SND(" ");
		SND("RageIRCd also supports \'search options\'. To use a search option, you must prefix the LIST");
		SND("command with \'/QUOTE\' to avoid your IRC Client interpreting the command. If you do not specify");
		SND("a search option, the server will display the full, unfiltered channel list.");
		SND(" ");
		SND("Search Options:");
		SND(" ");
		SND(" > number      - List channels with more than <number> of users");
		SND(" < number      - List channels with less than <nubmer> of users");
		SND(" C > number    - List channels created between now and <number> minutes ago");
		SND(" C < number    - List channels created earlier than <number> minutes ago");
		SND(" T > number    - List channels whose topics are older than <number> minutes");
		SND(" T < number    - List channels whose topics are newer than <number> minutes");
		SND(" *mask*        - List all channels that match *mask*");
		SND(" !*mask*       - List all channels that do not match *mask*");
		SND(" ");
		SND("Example: /LIST");
		SND("         /QUOTE LIST *IRCd*");
		SND("         /QUOTE LIST < 30");
	}
	else if (!myncmp(help, "PRIVMSG", 9) || !myncmp(help, "NOTICE", 9))
	{
		HDR("*** RageIRCd Help System - User Commands - PRIVMSG/NOTICE ***");
		SND(" ");
		SND("Used to sent text to clients and channels. PRIVMSG is used internally by IRC Clients");
		SND("for /MSG. RageIRCd also supports three additional formats.");
		SND(" ");
		SND("Example: /MSG [-Th3Dud3-] Hello. Im bored");
		SND("         /NOTICE [-Th3Dud3-] hey there! what are you doing?");
		SND("         /NOTICE @#channel :text    - Sends to all channel operators in #channel");
		SND("         /NOTICE %#channel :text    - Sends to all half operators in #channel");
		SND("         /NOTICE +#channel :text    - Sneds to all voiced users in #channel");
		SND(" ");
		SND("Note: The PRIVMSG syntax is the same as the NOTICE syntax above.");
	}
	else if (!myncmp(help, "KNOCK", 9))
	{
		HDR("*** RageIRCd Help System - User Commands - KNOCK ***");
		SND(" ");
		SND("Used to request to be invited into specified invite-only (+i) channel.");
		SND(" ");
		SND("Example: /KNOCK #Turbo-IRC Let me in, its cold outside!");
		SND("         /KNOCK #RageIRCd");
	}
	else if (!myncmp(help, "VHOST", 9))
	{
		HDR("*** RageIRCd Help System - User Commands - VHOST ***");
		SND(" ");
		SND("Provided you log in using the correct username and password, your hostname");
		SND("will be masked using the virtual host (vhost) provided by the IRC Server.");
		SND(" ");
		SND("Example: /VHOST on <login> <password>");
		SND("         /VHOST on billgates ilovelinux");
		SND("         /VHOST off");
	}
	else if (!myncmp(help, "TOPIC", 9))
	{
		HDR("*** RageIRCd Help System - User Commands - TOPIC ***");
		SND(" ");
		SND("Used to either display the current topic of the specified channel, or to change the");
		SND("of the specified channel to the specified topic.");
		SND(" ");
		SND("Example: /TOPIC #turbo-irc");
		SND("         /TOPIC #rageircd RageIRCd v1.5.0 has been released!");
	}
	else if (!myncmp(help, "MODE", 9))
	{
		HDR("*** RageIRCd Help System - User Commands - MODE ***");
		SND(" ");
		SND("Sets a mode upon a channel or user. For a full list of UserModes and their function,");
		SND("type \'/HELPOP USERMODES\'. For a full list of ChannelModes and their function, type");
		SND("\'/HELPOP CHANMODES\'.");
		SND(" ");
		SND("Example: /MODE [-Th3Dud3-] +z-h");
		SND("         /MODE #Turbo-IRC +mv-o [-Th3Dud3-] [-Th3Dud3-]");
	}
	else if (!myncmp(help, "INFO", 9))
	{
		HDR("*** RageIRCd Help System - User Commands - INFO ***");
		SND(" ");
		SND("Displays information about the IRC Server.");
		SND(" ");
		SND("Example: /INFO");
		SND("         /INFO MadChat.*");
	}
	else if (!myncmp(help, "VERSION", 9))
	{
		HDR("*** RageIRCd Help System - User Commands - VERSION ***");
		SND(" ");
		SND("Displays the version information about the current IRC Server software being used.");
		SND(" ");
		SND("Example: /VERSION");
		SND("         /VERSION Hub.*");
	}
	else if (!myncmp(help, "MOTD", 9))
	{
		HDR("*** RageIRCd Help System - User Commands - MOTD ***");
		SND(" ");
		SND("Displays the contents of the servers Message of the Day file (ircd.motd)");
		SND(" ");
		SND("Example: /MOTD");
		SND("         /MOTD DarkFyre.*");
	}
	else if (!myncmp(help, "SILENCE", 9))
	{
		HDR("*** RageIRCd Help System - User Commands - SILENCE ***");
		SND(" ");
		SND("Ignores messages from clients on your SILENCE list.");
		SND(" ");
		SND("Example: /SILENCE +nick    - adds a nickname to the SILENCE list");
		SND("         /SILENCE -nick    - deletes a nickname from the SILENCE list");
		SND("         /SILENCE          - displays the current SILENCE list");
	}
	else if (!myncmp(help, "WHOWAS", 9))
	{
		HDR("*** RageIRCd Help System - User Commands - WHOWAS ***");
		SND(" ");
		SND("Displays previous WHOIS information for specified users who are no longer connected");
		SND("to IRC.");
		SND(" ");
		SND("Example: /WHOWAS [-Th3Dud3-]");
		SND("         /WHOWAS Jamie");
	}
	else if (!myncmp(help, "NEWS", 9))
	{
		HDR("*** RageIRCd Help System - User Commands - NEWS ***");
		SND(" ");
		SND("Displays the contents of the IRC Servers NEWS file (ircd.news)");
		SND(" ");
		SND("Example: /NEWS");
		SND("         /NEWS Rage.*");
	}
	else if (!myncmp(help, "MKPASSWD", 9))
	{
		HDR("*** RageIRCd Help System - User Commands - MKPASSWD ***");
		SND(" ");
		SND("Encrypts the specified text so that it can be added directly to the ircd.conf file");
		SND("where crypt is enabled. Useful for adding IRC Operators with /ADDLINE.");
		SND(" ");
		SND("Example: /MKPASSWD 0p3rm3");
		SND("         /MKPASSWD myoperpassislreallylame");
	}
	else if (!myncmp(help, "PING", 9))
	{
		HDR("*** RageIRCd Help System - User Commands - PING ***");
		SND(" ");
		SND("The PING command is used to test the presence of an active client or server at");
		SND("the other end of the connection. Servers send a PING message at regular intervals");
		SND("if no other activity is detected from the connection. If the connection fails to");
		SND("reply to the PING message within a sertain amount of time, the connection is");
		SND("closed. A PING message may be sent even if the connection is active.");
		SND(" ");
		SND("Note: This function is not the same as the CTCP PING command.");
		SND(" ");
		SND("Example: PING <server1> <server2>");
		SND("         PING DarkFyre.*");
		SND("         PING [-Th3Dud3-]");
		SND("         PING [-Th3Dud3-] DarkFyre.*");
	}
	else if (!myncmp(help, "PONG", 9))
	{
		HDR("*** RageIRCd Help System - User Commands - PONG ***");
		SND(" ");
		SND("The PONG command is used to reply to a PING message. If the <server2> parameter");
		SND("is given, this message will be forwarded to the given target. The <server1>");
		SND("parameter is the name of the entity who has responded to the PING message using");
		SND("the PONG message.");
		SND(" ");
		SND("Example: PONG <server1> <server2>");
		SND("         PONG DarkFyre.* Hub.*     - PONG message from DarkFyre.* to Hub.*");
	}
	else if (!myncmp(help, "OPERCMDS", 9))
	{
		HDR("*** RageIRCd Help System - IRC Operator Commands ***");
		SND(" ");
		SND("Below is a list of IRC Operator Commands currently available. Type \'/HELPOP <command>\'");
		SND("for more information on the command.");
		SND(" ");
		SND("OPER      REHASH   AKILL    WALLOPS");
		SND("OPERCHAT  RESTART  RAKILL   GLINE");
		SND("ADCHAT    DIE      KILL     DYNSTATS");
		SND("TECHAT    ZLINE    SQUIT");
		SND("NACHAT    UNZLINE  CONNECT");
		SND("GLOBOPS   KLINE    ADDLINE");
		SND("LOCOPS    UNKLINE  TRACE");
	}
	else if (!myncmp(help, "OPER", 9))
	{
		HDR("*** RageIRCd Help System - IRC Operator Commands - OPER ***");
		SND(" ");
		SND("Sets IRC Operator privileges upon the user provided they \'OperUp\' correctly.");
		SND(" ");
		SND("Example: /OPER <name> <password>");
		SND("         /OPER th3dud3 0p3rm3");
	}
	else if (!myncmp(help, "REHASH", 9))
	{
		HDR("*** RageIRCd Help System - IRC Operator Commands - REHASH ***");
		SND(" ");
		SND("This command forces the IRC Server to re-load *ALL* the configuration files. If");
		SND("a server is specified, the specified server will re-load its configuration files.");
		SND(" ");
		SND("Example: /REHASH");
		SND("         /REHASH MadChat.*");
	}
	else if (!myncmp(help, "AKILL", 9))
	{
		HDR("*** RageIRCd Help System - IRC Operator Commands - AKILL ***");
		SND(" ");
		SND("Adds an AutoKill for the specified user@host. This will prevent any user matching");
		SND("the user@host from connecting to the network. Services Operators only.");
		SND(" ");
		SND("Example: AKILL foo@*.aol.com :Get a REAL ISP!");
	}
	else if (!myncmp(help, "WALLOPS", 9))
	{
		HDR("*** RageIRCd Help System - IRC Operator Commands - WALLOPS ***");
		SND(" ");
		SND("Sends a message to all users who have usermode +w. Only IRC Operators may send WALLOPS,");
		SND("however any user can set themselves +w and view WALLOP messages.");
		SND(" ");
		SND("Example: /WALLOPS Hello everyone who has mode +w set!");
	}
	else if (!myncmp(help, "OPERCHAT", 9))
	{
		HDR("*** RageIRCd Help System - IRC Operator Commands - OPERCHAT ***");
		SND(" ");
		SND("Sends an \'OperChat\' message to all IRC Operators with usermode +b. Only IRC");
		SND("Operators can send and receive if they have usermode +b. Used for general chat");
		SND("by IRC Operators, as a replacement for GLOBOPS which is used for important network");
		SND("information.");
		SND(" ");
		SND("The RageIRCd version of ChatOps!");
		SND(" ");
		SND("Example: /OPERCHAT :Hello to all the IRC Operators with usermode +b!");
	}
	else if (!myncmp(help, "RESTART", 9))
	{
		HDR("*** RageIRCd Help System - IRC Operator Commands - RESTART ***");
		SND(" ");
		SND("Forces the IRC Server to flush *ALL* of its connections and hash tables.");
		SND("Requires password if X:Line is present.");
		SND(" ");
		SND("Example: /RESTART");
	}
	else if (!myncmp(help, "RAKILL", 9))
	{
		HDR("*** RageIRCd Help System - IRC Operator Commands - RAKILL ***");
		SND(" ");
		SND("Used to remove an AutoKill set by an IRC Operator. Services Operators only.");
		SND(" ");
		SND("Example: /RAKILL foo@*.aol.com");
	}
	else if (!myncmp(help, "GLINE", 9))
	{
		HDR("*** RageIRCd Help System - IRC Operator Commands - GLINE ***");
		SND(" ");
		SND("Bans any user matching the user@hosts in the G:Line from connecting to any");
		SND("server on the network for a specified amount of time. If you specify the");
		SND("G:Line time as 0 seconds, the G:Line becomes permanent and will not expire.");
		SND(" ");
		SND("Example: /GLINE ADD <user@host> <time in seconds> :<reason>");
		SND("         /GLINE DEL <user@host>");
		SND("         /GLINE ADD *@*.aol.com 60 :AOL sucks - get a REAL ISP!");
		SND("                             - Bans *!*@*.aol.com for 60 seconds (1 minute)");
		SND("         /GLINE DEL *@*.aol.com");
	}
	else if (!myncmp(help, "ADCHAT", 9))
	{
		HDR("*** RageIRCd Help System - IRC Operator Commands - ADCHAT ***");
		SND(" ");
		SND("Used to send a message to all Server Administrators and Co-Server");
		SND("Administrators connected to IRC. Co-Server Administrators and up only.");
		SND(" ");
		SND("Example: /ADCHAT :Hello all the Admins!");
	}
	else if (!myncmp(help, "DIE", 9))
	{
		HDR("*** RageIRCd Help System - IRC Operator Commands - DIE ***");
		SND(" ");
		SND("Used to terminate (shut down) the IRC Server. Requires password if X:Line");
		SND("is present.");
		SND(" ");
		SND("Example: /DIE");
	}
	else if (!myncmp(help, "KILL", 9))
	{
		HDR("*** RageIRCd Help System - IRC Operator Commands - KILL ***");
		SND(" ");
		SND("Removes the specified user(s) from the server.");
		SND(" ");
		SND("Example: /KILL <user1>,<user2>,<user3> <reason>");
		SND("         /KILL [-Th3Dud3-],Jamie,kmw goodbye lamers!");
	}
	else if (!myncmp(help, "TECHAT", 9))
	{
		HDR("*** RageIRCd Help System - IRC Operator Commands - TECHAT ***");
		SND(" ");
		SND("Used to send a message to all Technical Administrators connected to IRC.");
		SND("Technical Administrators and up only.");
		SND(" ");
		SND("Example: /TECHAT :Hello all the TechAdmins!");
	}
	else if (!myncmp(help, "ZLINE", 9))
	{
		HDR("*** RageIRCd Help System - IRC Operator Commands - ZLINE ***");
		SND(" ");
		SND("Bans all users matching the specified IP address. Note: The ban is *NOT* global.");
		SND(" ");
		SND("Example: /ZLINE 127.0.0.1 :We do not allow connections from localhost");
	}
	else if (!myncmp(help, "SQUIT", 9))
	{
		HDR("*** RageIRCd Help System - IRC Operator Commands - SQUIT ***");
		SND(" ");
		SND("Terminates the connection between an IRC Server on a network.");
		SND(" ");
		SND("Example: /SQUIT HiTech.*");
	}
	else if (!myncmp(help, "NACHAT", 9))
	{
		HDR("*** RageIRCd Help System - IRC Operator Commands - NACHAT ***");
		SND(" ");
		SND("Used to send a message to all Co-Network Administrators and Network Administrators");
		SND("connected to IRC. Co-Network and Network Administrators only.");
		SND(" ");
		SND("Example: /NACHAT :Hello to all the NetAdmins!");
	}
	else if (!myncmp(help, "UNZLINE", 9))
	{
		HDR("*** RageIRCd Help System - IRC Operator Commands - UNZLINE ***");
		SND(" ");
		SND("Used to remove a Z:Line.");
		SND(" ");
		SND("Example: /UNZLINE 127.0.0.1");
	}
	else if (!myncmp(help, "CONNECT", 9))
	{
		HDR("*** RageIRCd Help System - IRC Operator Commands - CONNECT ***");
		SND(" ");
		SND("Links an IRC Server to the server you are currently on. RageIRCd also allows");
		SND("\'remote\' connections.");
		SND(" ");
		SND("Example: /CONNECT <server>");
		SND("         /CONNECT <server1> <port> <server2>");
		SND("         /CONNECT Hub.*");
		SND("         /CONNECT MadChat.* 7000 Hub.*");
	}
	else if (!myncmp(help, "GLOBOPS", 9))
	{
		HDR("*** RageIRCd Help System - IRC Operator Commands - GLOBOPS ***");
		SND(" ");
		SND("Sends a message to all IRC Operators connected to the network. Unlike WALLOPS,");
		SND("the messages are only viewable by IRC Operators. Normally used for important");
		SND("network information.");
		SND(" ");
		SND("Example: /GLOBOPS :Them damn flooders are back!");
	}
	else if (!myncmp(help, "DYNSTATS", 9))
	{
		HDR("*** RageIRCd Help System - IRC Operator Commands - DYNSTATS ***");
		SND(" ");
		SND("Displays the current Dynamic Configuration settings.");
		SND(" ");
		SND("Example: /DYNSTATS");
	}
	else if (!myncmp(help, "KLINE", 9))
	{
		HDR("*** RageIRCd Help System - IRC Operator Commands - KLINE ***");
		SND(" ");
		SND("Bans the specified user@host from the IRC Server. Note: The ban is *NOT* global.");
		SND(" ");
		SND("Example: /KLINE *@*.btinternet.com Abuse");
	}
	else if (!myncmp(help, "ADDLINE", 9))
	{
		HDR("*** RageIRCd Help System - IRC Operator Commands - ADDLINE ***");
		SND(" ");
		SND("Adds a line of text to the specified file, provided the file exists. Co-Server");
		SND("Administrators and up only.");
		SND(" ");
		SND("Example: /ADDLINE <filename> :<line of text to add>");
		SND("         /ADDLINE config/vhost.conf :vhost billgates ilovelinux microsoft.com");
	}
	else if (!myncmp(help, "LOCOPS", 9))
	{
		HDR("*** RageIRCd Help System - IRC Operator Commands - LOCOPS ***");
		SND(" ");
		SND("Similar to GLOBOPS, accept only IRC Operators local to your server receive");
		SND("the message.");
		SND(" ");
		SND("Example: /LOCOPS :Hello to all the IRC Operators on my server!");
	}
	else if (!myncmp(help, "UNKLINE", 9))
	{
		HDR("*** RageIRCd Help System - IRC Operator Commands - UNKLINE ***");
		SND(" ");
		SND("Used to remove a K:Line from the server.");
		SND(" ");
		SND("Example: /UNKLINE *@*.btinternet.com");
	}
	else if (!myncmp(help, "TRACE", 9))
	{
		HDR("*** RageIRCd Help System - IRC Operator Commands - TRACE ***");
		SND(" ");
		SND("Used to find out what servers are connected to what.");
		SND(" ");
		SND("Example: /TRACE DarkFyre.*");
	}
	else if (!myncmp(help, "USERMODES", 9))
	{
		HDR("*** RageIRCd Help System - User Modes ***");
		SND(" ");
		SND("Below is a list of the User Modes currently available, along with their function.");
		SND(" ");
		SND("N - Network Administrator");
		SND("C - Co-Network Administrator");
		SND("T - Technical Administrator");
		SND("Q - Abuse Administrator");
		SND("a - Services Operator");
		SND("A - Server Administrator");
		SND("J - Co-Server Administrator");
		SND("o - IRC Operator (Global Oper)");
		SND("O - Server Operator (Local Oper)");
		SND("h - Help Operator");
		SND("i - Invisible (will not appear on /WHO lists");
		SND("g - Can send/receive LOCOP and GLOBOP notices");
		SND("s - Can listen to server notices");
		SND("k - Can listen to KILL notices");
		SND("c - Can see connecting/exiting clients");
		SND("f - Can listen to flood alerts");
		SND("r - Registered Nickname");
		SND("z - Masked hostname");
		SND("b - User can send/receive OPERCHAT notices");
		SND("B - Bot");
		SND("L - Can listen to Global Connection notices");
		SND("S - Protected/Services Client (U:Lines only)");
		SND("W - Can see clients /whois'ing them");
		SND("w - Can send/receive WALLOP messages");
		SND("H - Services Root Administrator");
		SND("d - DCC Block (blocks B:Lined files from being sent to user");
		SND("V - Badword Filter (replaces \'badwords\' with \'<censored>\'");
		SND(" ");
		SND("To set a usermode on yourself, type: \'/mode <yournick> <modes>\'");
		SND("Example: /mode [-Th3Dud3-] +zd");
		SND("         /mode [-Th3Dud3-] -zd");
	}
	else if (!myncmp(help, "CHANMODES", 9))
	{
		HDR("*** RageIRCd Help System - Channel Modes ***");
		SND(" ");
		SND("Below is a list of the Channel Modes currently available, along with their function.");
		SND(" ");
		SND("p - Private (will not show on /LIST; cannot join unless invited)");
		SND("s - Secret (will not show on /LIST");
		SND("i - Invite Only (cannot join unless /INVITE'd)");
		SND("K - No Knocks (dissables /KNOCK for specific channel)");
		SND("m - Moderated (only op/halfop/voice users may speak)");
		SND("n - No External Messages");
		SND("t - Topic Lock (only op/halfop users may change topic)");
		SND("r - Registered Channel");
		SND("R - Registered Only (only registered nicknames may join)");
		SND("c - No Colors (blocks all color from the channel)");
		SND("A - Administrators Only");
		SND("O - IRC Operators Only");
		SND("j - Peace Mode (dissables the /KICK function)");
		SND("d - No Nickname Changes");
		SND("N - Network Administrators Only");
		SND("T - No Invites (dissables the /INVITE function)");
		SND("S - Strip Colors (all colors are stripped from text)");
		SND("V - Strip Badwords (all badwords are replaced with \'<censored>\')");
		SND("x - Strip Formatting (all formatting [excl. color] is stripped from text)");
		SND("C - No CTCPs (blocks all /CTCP requests to the channel)");
		SND("B - Operator Ban (dissables OperWalk)");
		SND("a <nickname> - Channel Admin (cannot be deopped/halfopped/voiced/kicked");
		SND("               from channel accept by services)");
		SND("o <nickname> - Channel Operator (can toggle channel settings)");
		SND("h <nickname> - Half Operator (can toggle various channel settings)");
		SND("v <nickname> - Voiced User (can speak even when banned or channel is +m)");
		SND("l <number> - User Limit (only specified amount of users may join channel)");
		SND("k <key> - 'Lock' Channel (must specify correct key to join channel)");
		SND("L <channel> - Linked Channels (users get bounced to specified channel if");
		SND("              user limit is reached)");
		SND("f [*]<lines>:<seconds> - Flood Limit (user is kicked if specified amount of");
		SND("                         lines are sent within specified amount of seconds;");
		SND("                         If * is specified, user is also banned)");
		SND("b <nick!user@host> - Ban (no users matching nick!user@host may join channel)");
		SND("e <nick!user@host> - Exception Ban (users matching nick!user@host may join channel");
		SND("                     even when banned)");
		SND("F <nick!user@host>:<channel> - Forwarding Ban (users matching nick!user@host are");
		SND("                               forwarded to specified channel)");
		SND(" ");
		SND("To set a channel mode, type: \'/mode #channel <mode> [parameters]\'");
		SND("Example: /mode #turbo-irc +ntSo [-Th3Dud3-]");
		SND("         /mode #RageIRCd +F *kmw*!*@*.bt.com:#goaway");
	}
	else if (!myncmp(help, "OFLAGS", 9))
	{
		SND("*** RageIRCd Help System - O:Line Flags ***");
		SND(" ");
		SND("Below is a list of the O:Line Flags currently available, along with their function.");
		SND(" ");
		SND("N - Network Administrator (auto +N on oper)");
		SND("C - Co-Network Administrator (auto +C on oper)");
		SND("T - Technical Administrator (auto +T on oper)");
		SND("Q - Abuse Administrator (auto +Q on oper)");
		SND("a - Services Operator (auto +a on oper)");
		SND("A - Server Administrator (auto +A on oper)");
		SND("J - Co-Server Administrator (auto +J on oper)");
		SND("O - IRC Operator (Global Oper)");
		SND("o - Server Operator (Local Oper)");
		SND("h - Help Operator (auto +h on oper)");
		SND("r - Oper can /REHASH");
		SND("R - Oper can /RESTART");
		SND("D - Oper can /DIE");
		SND("l - Oper can /LOCOPS");
		SND("g - Oper can /GLOBOPS");
		SND("c - Oper can do local routing");
		SND("U - Oper can do global routing");
		SND("L - Oper can listen to Global Connection notices (auto +L on oper)");
		SND("k - Oper can do Local /KILL's");
		SND("K - Oper can do Global /KILL's");
		SND("b - Oper can /KLINE");
		SND("B - Oper can /UNKLINE");
		SND("n - Oper can send Local Notices (/notice $server.irc.net <message>)");
		SND("G - Oper can send Global Notices (/notice $*.irc.net <message>)");
		SND("u - Oper can set self +c");
		SND("f - Oper can set self +f");
		SND("z - Oper can /ZLINE and /UNZLINE");
	}
	else if (!myncmp(help, "ABOUT", 9))
	{
		HDR("*** RageIRCd Help System - About ***");
		SND(" ");
		SND("The RageIRCd Help System was coded by [-Th3Dud3-] (aka Alasdair McWilliam)");
		SND("as a replacement for documentation files. For more information on the IRCd");
		SND("itself, type \'/INFO\'");
		SND(" ");
		SND("The RageIRCd Help System - Copyright (C) Alasdair McWilliam. RageIRCd, Inc.");
	}
	else
	{
		HDR("*** RageIRCd Help System ***");
		HLP(" ");
		HLP("Please specify a help topic after the /HELPOP command from the list below.");
		HLP("If your query is not within any of the help topics below, you can forward it");
		HLP("to Help Operators by prefixing a \'!\' to the query. Prefixing a \'?\' to the");
		HLP("query will force it to be sent only to the RageIRCd Help System.");
		HLP(" ");
		HLP("Example: /HELPOP :!how do you join a channel");
		HLP("         /HELPOP ?usercmds");
		HLP(" ");
		HLP("/HELPOP USERCMDS     - A list of User commands");
		HLP("/HELPOP OPERCMDS     - A list of IRC Operator commands");
		HLP("/HELPOP USERMODES    - A list of UserModes and their functions");
		HLP("/HELPOP CHANMODES    - A list of ChannelModes and their functions");
		HLP("/HELPOP OFLAGS       - A list of O:Line flags and their functions");
		HLP("/HELPOP ABOUT        - Information about the RageIRCd Help System");
		HLP(" ");
		HLP("If you require assistance with Services:");
		HLP(" ");
		sendto_one(sptr, ":%s 293 %s :    /msg %s HELP    - Channel Services Information", me.name, name, ChanServ);
		sendto_one(sptr, ":%s 293 %s :    /msg %s HELP    - Nickname Services Information", me.name, name, NickServ);
		sendto_one(sptr, ":%s 293 %s :    /msg %s HELP    - Memo Services Information", me.name, name, MemoServ);
		sendto_one(sptr, ":%s 293 %s :    /msg %s HELP    - Operator Services Information", me.name, name, OperServ);
	}
	HLP(" ");
	sendto_one(sptr, ":%s 292 %s :If you require any further assistance, please join %s.", me.name, name, ircnet_channel);
	HLP("-");
	return 1;
}
