/************************************************************************
 * Unreal Internet Relay Chat Daemon, src/ircstats.c
 * (C) Carsten Munk <stskeeps@tspre.org> 2000
 *
 * Modified for RageIRCd by Alasdair McWilliam <th3dud3@turbo-irc.net>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 1, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include "struct.h"
#include "common.h"
#include "sys.h"
#include "numeric.h"
#include "msg.h"
#include "channel.h"
#include "userload.h"
#include "patchlevel.h"
#include "ircstats.h"
#include <time.h>
#include <sys/stat.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <utmp.h>
#include <fcntl.h>
#include "h.h"

ircstats IRCstats;

#ifdef IRC_COMMENT
int	clients;
int	invisible;
int	servers;
int	operators;
int	unknown;
int	channels;
int	me_clients;
int	me_servers;
int	me_max;
int	global_max;
#endif

void init_ircstats(void)
{
	bzero(&IRCstats, sizeof(IRCstats));
	IRCstats.servers = 1;
}

void save_stats(void)
{
	FILE *statsfile = fopen(IRCSTATS, "w+");

	if (!statsfile)
	{
		sendto_ops("Cannot find %s - unable to save current IRCstats.", IRCSTATS);
		return;
	}

	fprintf(statsfile, "%d\n", IRCstats.me_max);
	fprintf(statsfile, "%d\n", IRCstats.global_max);

	fclose(statsfile);
}

int load_statsfile(char *file)
{
	FILE *statsfile = fopen(IRCSTATS, "r");
	char buf[1024];

	if (!statsfile)
		return -1;

	/* load max local users */
	fgets(buf, 1024, statsfile);
	IRCstats.me_max = atoi(buf);

	/* load max global users */
	fgets(buf, 1024, statsfile);
	IRCstats.global_max = atoi(buf);

	fclose(statsfile);
	return 0;
}
