/************************************************************************
 * IRC - Internet Relay Chat, src/s_rage.c
 * Copyright (C) 2000 Alasdair McWilliam. RageIRCd, Inc.
 *
 * Contains code from UnrealIRCd - Copyright (C) The UnrealIRCd Team.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 1, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include "struct.h"
#include "common.h"
#include "sys.h"
#include "numeric.h"
#include "msg.h"
#include "channel.h"
#include "userload.h"
#if defined(PCS) || defined(AIX) || defined(DYNIXPTX) || defined(SVR3)
#include <time.h>
#endif
#include <sys/stat.h>
#include <fcntl.h>
#include <utmp.h>
#include <time.h>
#include "h.h"
#include "dynconf2.h"

/*
 * m_operchat - sends text to operators who are +b
 *
 * parv[0] - sender prefix
 * parv[1] - message text
 *
 * Revamped. -TD (December 2000)
 * Re-revamped. -TD (March 2001)
 */
int	m_operchat(cptr, sptr, parc, parv)
	aClient		*cptr, *sptr;
	int		parc;
	char		*parv[];
{
	char		*message;

	if (ENABLE_OPERCHAT == TRUE)
	{
		if (MyClient(sptr) && !IsServer(sptr) && !IsOperChat(sptr))
		{
			sendto_one(sptr, err_str(ERR_NOPRIVILEGES), me.name, parv[0]);
			return 0;
		}
	}
	else
	{
		if (MyClient(sptr) && !IsServer(sptr))
		{
			sendto_one(sptr, err_str(ERR_COMMANDDISSABLED), me.name, parv[0], "OPERCHAT");
			return 0;
		}
	}

	message = parc > 1 ? parv[1] : NULL;
	if (BadPtr(message))
	{
		sendto_one(sptr, err_str(ERR_NEEDMOREPARAMS), me.name, parv[0], "OPERCHAT");
		return 0;
	}
	sendto_serv_butone(IsServer(sptr) ? cptr : NULL, ":%s OPERCHAT :%s", parv[0], message);
	sendto_umode(UMODE_OPERCHAT, "*** OperChat -- from %s: %s", parv[0], message);

	return 0;
}

/*
 * m_adchat - admin chat
 *
 * parv[0] - sender prefix
 * parv[1] - message text
 *
 * Revamped. -TD (December 2000)
 * Re-revamped. -TD (March 2001)
 */
int	m_adchat(cptr, sptr, parc, parv)
	aClient		*cptr, *sptr;
	int		parc;
	char		*parv[];
{
	char		*message;

	if (ENABLE_ADCHAT == TRUE)
	{
		if (MyClient(sptr) && !IsServer(sptr) && !(IsAdmin(sptr) || IsSCoAdmin(sptr)))
		{
			sendto_one(sptr, err_str(ERR_NOPRIVILEGES), me.name, parv[0]);
			return 0;
		}
	}
	else
	{
		if (MyClient(sptr) && !IsServer(sptr))
		{
			sendto_one(sptr, err_str(ERR_COMMANDDISSABLED), me.name, parv[0], "ADCHAT");
			return 0;
		}
	}

	message = parc > 1 ? parv[1] : NULL;
	if (BadPtr(message))
	{
		sendto_one(sptr, err_str(ERR_NEEDMOREPARAMS), me.name, parv[0], "ADCHAT");
		return 0;
	}
	sendto_serv_butone(IsServer(sptr) ? cptr : NULL, ":%s ADCHAT :%s", parv[0], message);
	sendto_umode(UMODE_ADMIN, "*** AdminChat -- from %s: %s", parv[0], message);
	sendto_umode(UMODE_SCOADMIN, "*** AdminChat -- from %s: %s", parv[0], message);

	return 0;
}

/*
 * m_techat - techadmin chat
 *
 * parv[0] - sender prefix
 * parv[1] - message text
 *
 * Revamped. -TD (December 2000)
 * Re-revamped. -TD (March 2001)
 */
int	m_techat(cptr, sptr, parc, parv)
	aClient		*cptr, *sptr;
	int		parc;
	char	*parv[];
{
	char    *message;

	if (ENABLE_TECHAT == TRUE)
	{
		if (MyClient(sptr) && !IsServer(sptr) && !IsTechAdmin(sptr))
		{
			sendto_one(sptr, err_str(ERR_NOPRIVILEGES), me.name, parv[0]);
			return 0;
		}
	}
	else
	{
		if (MyClient(sptr) && !IsServer(sptr))
		{
			sendto_one(sptr, err_str(ERR_COMMANDDISSABLED), me.name, parv[0], "TECHAT");
			return 0;
		}
	}

	message = parc > 1 ? parv[1] : NULL;
	if (BadPtr(message))
	{
		sendto_one(sptr, err_str(ERR_NEEDMOREPARAMS), me.name, parv[0], "TECHAT");
		return 0;
	}
	sendto_serv_butone(IsServer(sptr) ? cptr : NULL, ":%s TECHAT :%s", parv[0], message);
	sendto_umode(UMODE_TECHADMIN, "*** TechAdminChat -- from %s: %s", parv[0], message);

	return 0;
}

/*
 * m_nachat -- netadmin chat
 *
 * parv[0] - sender prefix
 * parv[1] - message text
 *
 * Revamped. -TD (December 2000)
 * Re-revamped. -TD (March 2001)
 */
int	m_nachat(cptr, sptr, parc, parv)
	aClient		*cptr, *sptr;
	int		parc;
	char		*parv[];
{
	char		*message;

	if (ENABLE_NACHAT == TRUE)
	{
		if (MyClient(sptr) && !IsServer(sptr) && !(IsNetAdmin(sptr) || IsCoNetAdmin(sptr)))
		{
			sendto_one(sptr, err_str(ERR_NOPRIVILEGES), me.name, parv[0]);
			return 0;
		}
	}
	else
	{
		if (MyClient(sptr) && !IsServer(sptr))
		{
			sendto_one(sptr, err_str(ERR_COMMANDDISSABLED), me.name, parv[0], "NACHAT");
			return 0;
		}
	}

	message = parc > 1 ? parv[1] : NULL;
	if (BadPtr(message))
	{
		sendto_one(sptr, err_str(ERR_NEEDMOREPARAMS), me.name, parv[0], "NACHAT");
		return 0;
	}
	sendto_serv_butone(IsServer(sptr) ? cptr : NULL, ":%s NACHAT :%s", parv[0], message);
	sendto_umode(UMODE_NETADMIN, "*** NetAdminChat -- from %s: %s", parv[0], message);
	sendto_umode(UMODE_CONETADMIN, "*** NetAdminChat -- from %s: %s", parv[0], message);

	return 0;
}

/*
 * m_abuse
 *
 *	parv[0] = sender prefix
 *	parv[1] = message
 */
int	m_abuse(cptr, sptr, parc, parv)
	aClient		*cptr, *sptr;
	int		parc;
	char		*parv[];
{
	char		*message;

	message = parc > 1 ? parv[1] : NULL;
	if (BadPtr(message))
	{
		sendto_one(sptr, err_str(ERR_NEEDMOREPARAMS), me.name, parv[0], "ABUSE");
		return 0;
	}
	sendto_serv_butone(IsServer(sptr) ? cptr : NULL, ":%s ABUSE :%s", parv[0], message);
	sendto_umode(UMODE_ABUSEADMIN, "*** Abuse -- from %s: %s", parv[0], message);

	if (MyClient(sptr))
	{
		sendto_one(sptr, ":%s NOTICE %s :*** Your abuse message has been forwarded to the Abuse Administrators",
			me.name, parv[0]);
	}
	return 0;
}

/*
 * m_addline - adds text to the specified file
 *
 * parv[0] - sender prefix
 * parv[1] - file to write to
 * parv[3] - text to write
 *
 * Revamped. -TD (December 2000)
 * Re-revamped. -TD (March 2001)
 */
int	m_addline(cptr, sptr, parc, parv)
	aClient		*cptr, *sptr;
	int		parc;
	char		*parv[];
{
	FILE		*fd;
	char		*text;

	if (ENABLE_ADDLINE == TRUE)
	{
		if (MyClient(sptr) && !IsServer(sptr) && !(IsAdmin(sptr) || IsSCoAdmin(sptr)))
		{
			sendto_one(sptr, err_str(ERR_NOPRIVILEGES), me.name, parv[0]);
			return 0;
		}
	}
	else
	{
		if (MyClient(sptr) && !IsServer(sptr))
		{
			sendto_one(sptr, err_str(ERR_COMMANDDISSABLED), me.name, parv[0], "ADDLINE");
			return 0;
		}
	}

	if (parc < 3)
	{
		sendto_one(sptr, err_str(ERR_NEEDMOREPARAMS), me.name, parv[0], "ADDLINE");
		return 0;
	}

	text = parc > 2 ? parv[2] : NULL;
	if (BadPtr(text))
	{
		sendto_one(sptr, ":%s NOTICE %s :*** AddLine -- No text to write", me.name, parv[0]);
		return 0;
	}

	fd = fopen(parv[1], "r");
	if (!fd)
	{
		sendto_one(sptr, ":%s NOTICE %s :*** AddLine -- Unable to open %s", me.name, parv[0],
			parv[1]);
		return 0;
	}
	else
	{
		fclose(fd);
		fd = fopen(parv[1], "a");
	}

	(void)fprintf(fd, "%s\n", text);

	sendto_one(sptr, ":%s NOTICE %s :*** AddLine -- Written: %s to: %s", me.name, parv[0], text, parv[1]);
	sendto_ops("%s (%s@%s) has written to file: %s", sptr->name, sptr->user->username,
		IsMasked(sptr) ? sptr->user->maskedhost : sptr->user->host, parv[1]);

	fclose(fd);
	return 0;
}

/*
 * m_newident - changes current ident
 *
 * parv[0] - sender prefix
 * parv[1] - new ident
 *
 * Revamped. -TD (December 2000)
 * Re-revamped. -TD (March 2001)
 */
int	m_newident(cptr, sptr, parc, parv)
	aClient		*cptr, *sptr;
	int		parc;
	char		*parv[];
{
	char		*newident, *s;
	int		badident = 0;

	if (ENABLE_NEWIDENT == FALSE)
	{
		if (MyClient(sptr) && !IsServer(sptr))
		{
			sendto_one(sptr, err_str(ERR_COMMANDDISSABLED), me.name, parv[0], "NEWIDENT");
			return 0;
		}
	}

	newident = parc > 1 ? parv[1] : NULL;
	if (BadPtr(newident))
	{
		sendto_one(sptr, ":%s NOTICE %s :*** NewIdent -- Syntax: /newident <newident>", me.name,
			parv[0]);
		return 0;
	}

	if (strlen(newident) < 1)
	{
		sendto_one(sptr, ":%s NOTICE %s :*** NewIdent -- Syntax: /newident <newident>", me.name,
			parv[0]);
		return 0;
	}

	if (strlen(newident) > (USERLEN - 1))
	{
		sendto_one(sptr, ":%s NOTICE %s :*** NewIdent -- Ident too long (max. %i characters)",
			me.name, parv[0], (USERLEN - 1));
		return 0;
	}

	for (s = newident; *s; s++)
	{
		if (!isallowed(*s))
		{
			badident = 1;
		}
		if (*s == '~')
		{
			badident = 0;
		}
	}

	if (badident == 1)
	{
		sendto_one(sptr, ":%s NOTICE %s :*** NewIdent -- Ident contains illegal characters! Use"
			" only characters a-z, A-Z, 0-9, - . and ~", me.name, parv[0]);
		return 0;
	}

	sendto_ops("%s (%s@%s) has changed their ident (old: %s new: %s)", sptr->name, sptr->user->username,
		IsMasked(sptr) ? sptr->user->maskedhost : sptr->user->host, sptr->user->username, newident);
	if (MyClient(sptr))
	{
		sendto_one(sptr, ":%s NOTICE %s :*** NewIdent -- Your ident has been changed (old: %s new: %s)",
			me.name, parv[0], sptr->user->username, newident);
	}

	(void)ircsprintf(sptr->user->username, "%s", newident);
	sendto_serv_butone(cptr, ":%s NEWIDENT %s", parv[0], newident);

	return 0;
}

/*
 * m_newname - changes current name (user info)
 *
 * parv[0] - sender prefix
 * parv[1] - new name
 *
 * Revamped. -TD (December 2000)
 * Re-revamped. -TD (March 2001)
 */
int	m_newname(cptr, sptr, parc, parv)
	aClient		*cptr, *sptr;
	int		parc;
	char		*parv[];
{
	char		*newname;

	if (ENABLE_NEWNAME == FALSE)
	{
		if (MyClient(sptr) && !IsServer(sptr))
		{
			sendto_one(sptr, err_str(ERR_COMMANDDISSABLED), me.name, parv[0], "NEWNAME");
			return 0;
		}
	}

	newname = parc > 1 ? parv[1] : NULL;
	if (BadPtr(newname))
	{
		sendto_one(sptr, ":%s NOTICE %s :*** NewName -- Syntax: /newname :<new name>", me.name,
			parv[0]);
		return 0;
	}

	if (strlen(newname) < 1)
	{
		sendto_one(sptr, ":%s NOTICE %s :*** NewName -- Syntax: /newname :<new name>", me.name,
			parv[0]);
		return 0;
	}

	if (strlen(newname) > (REALLEN - 1))
	{
		sendto_one(sptr, ":%s NOTICE %s :*** NewName -- Realname too long (max. %i chars)",
			me.name, parv[0], (REALLEN - 1));
		return 0;
	}

	if (find_bannedname(newname))
	{
		sendto_one(sptr, ":%s NOTICE %s :*** NewName -- The name you specified is not permitted on this server (n:Lined).",
			me.name, parv[0]);
		return 0;
	}

	sendto_ops("%s (%s@%s) has changed their realname (old: %s new: %s)", sptr->name, sptr->user->username,
		IsMasked(sptr) ? sptr->user->maskedhost : sptr->user->host, sptr->info, newname);
	if (MyClient(sptr))
	{
		sendto_one(sptr, ":%s NOTICE %s :*** NewName -- Your realname has been changed (old: %s new: %s)",
			me.name, parv[0], sptr->info, newname);
	}

	(void)ircsprintf(sptr->info, "%s", newname);
	sendto_serv_butone(cptr, ":%s NEWNAME :%s", parv[0], newname);

	return 0;
}

/*
 * m_svschghost - changes the hostname of specified user to specified host.
 *                For compatibility with Auspice Services and 'HostServ',
 *                based on UnrealIRCd's m_chghost().
 *                                                          -TD (04/12/00)
 *
 * parv[0] - sender prefix
 * parv[1] - user to change hostname of
 * parv[2] - hostname to change to
 */
int	m_svschghost(cptr, sptr, parc, parv)
	aClient		*cptr, *sptr;
	int		parc;
	char		*parv[];
{
	aClient		*acptr;
	char		*s, *hostname;
	int		blockchange = 0;

	if (IsPerson(sptr) && !IsULine(cptr, sptr) && !IsServer(cptr))
		return 0;

	if ((parc < 3) || (parc > 3))
		return 0;

	if (strlen(parv[2]) < 1)
		return 0;

	if (strlen(parv[2]) > (HOSTLEN - 1))
		return 0;

	for (s = parv[2]; *s; s++)
	{
		if (!isallowed(*s))
		{
			blockchange = 1;
		}
	}
	if (blockchange == 1)
		return 0;

	hostname = parv[2];
	if (hostname == NULL)
		return 0;

	if ((acptr = find_person(parv[1], NULL)))
	{
		acptr->umodes |= UMODE_MASK;
		strncpyzt(acptr->user->maskedhost, hostname, HOSTLEN);
		sendto_serv_butone(cptr, ":%s SVSCHGHOST %s :%s", parv[0], parv[1], hostname);
	}
	return 0;
}

/*
 * m_mkpasswd - encrypts text
 *
 * parv[0] - sender prefix
 * parv[1] - text to encrypt
 */
int	m_mkpasswd(cptr, sptr, parc, parv)
	aClient		*cptr, *sptr;
	int		parc;
	char		*parv[];
{
#ifdef CRYPT_OPER_PASSWORD
	static	char	saltChars[] = "abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789./";
	extern	char	*crypt();
	char		salt[3];
	int		usable = 0;

	if (parc > 1)
	{
		if (strlen(parv[1]) >= 1)
		{
			usable = 1;
		}
	}

	if (usable == 0)
	{
		sendto_one(sptr, ":%s NOTICE %s :*** Text to encrypt must be atleast one character in length", me.name, parv[0]);
		return 0;
	}

	srandom(time(0));
	salt[0] = saltChars[random() % 64];
	salt[1] = saltChars[random() % 64];
	salt[2] = 0;

	if (((char *)strchr((char *)saltChars, salt[0]) == NULL) || ((char *)strchr((char *)saltChars, salt[1]) == NULL))
	{
		sendto_one(sptr, ":%s NOTICE %s :*** Illegal encryption salt - %s", me.name, parv[0], salt);
		return 0;
	}

	sendto_one(sptr, ":%s NOTICE %s :*** Encryption for [%s]: %s", me.name, parv[0], parv[1], crypt(parv[1], salt));
#else
	sendto_one(sptr, ":%s NOTICE %s :*** Sorry, encryption has not been enabled on this server.", me.name, parv[0]);
#endif /* CRYPT_OPER_PASSWORD */
	return 0;
}
